import random
from typing import Tuple

from sqlalchemy import select, insert, ScalarResult, update, delete
from sqlalchemy.orm.attributes import flag_modified

from ..database.db import async_session
from ..database.models import Contact, GroupChat, UserProfile

pool = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"]


def _create_random_id() -> str:
    random_chars = random.choices(pool, k=11)
    return "".join(random_chars)


async def insert_group_chat(supervisor: str, members: list[str]) -> GroupChat:
    id = _create_random_id()
    name = f"群聊 ({id})"
    session = async_session()
    try:
        group_chat_res = await session.scalars(
            insert(GroupChat)
            .values(id=id, name=name, supervisor=supervisor, members=members)
            .returning(GroupChat)
        )
        contact_res: ScalarResult[Contact] = await session.scalars(
            select(Contact).where(Contact.user_id.in_(members))
        )
        for contact in contact_res.all():
            contact.group_chats[id] = {"groupChatRemark": "", "remarkInGroupChat": ""}
            flag_modified(contact, "group_chats")

        session.add_all(contact_res.all())
        await session.commit()
        return group_chat_res.one()
    except Exception as e:
        raise e
    finally:
        await session.close()


async def insert_group_chat_members(group_chat_id: str, members: list[str]):
    session = async_session()
    try:
        group_chat: GroupChat = (
            await session.scalars(
                select(GroupChat).where(GroupChat.id == group_chat_id)
            )
        ).one()
        group_chat.members.extend(members)
        flag_modified(group_chat, "members")
        session.add(group_chat)

        for member in members:
            contact: Contact = (
                await session.scalars(select(Contact).where(Contact.user_id == member))
            ).one()
            contact.group_chats[group_chat_id] = {
                "remarkInGroupChat": "",
                "groupChatRemark": "",
            }
            flag_modified(contact, "group_chats")
            session.add(contact)

        await session.commit()
    except Exception as e:
        raise e
    finally:
        await session.close()


async def select_member_name_avatar(member_id: str, is_friend: bool) -> dict | Tuple:
    session = async_session()
    if is_friend:
        res: ScalarResult[dict] = await session.scalars(
            select(Contact.group_chats).where(Contact.user_id == member_id)
        )
        await session.close()
        # {'81906574618': {'remarkInGroupChat': '', 'groupChatRemark': ''}}
        return res.one()
    else:
        res = await session.execute(
            select(UserProfile.nickname, UserProfile.avatar, Contact.group_chats)
            .join(Contact, UserProfile.user_id == Contact.user_id)
            .where(UserProfile.user_id == member_id)
        )
        await session.close()
        # ('htylight', 'cznowoyn1692502503.png', {'81906574618': {'remarkInGroupChat': '', 'groupChatRemark': ''},})
        return res.one()


async def select_full_profile(group_chat_id: str) -> Tuple[GroupChat, list[Tuple[str]]]:
    session = async_session()
    group_chat: GroupChat = (
        await session.scalars(select(GroupChat).where(GroupChat.id == group_chat_id))
    ).one()
    members: list[str] = group_chat.members
    res = await session.execute(
        select(
            UserProfile.user_id,
            UserProfile.nickname,
            UserProfile.avatar,
            Contact.group_chats[group_chat_id]["remarkInGroupChat"],
        )
        .join(Contact, UserProfile.user_id == Contact.user_id)
        .where(UserProfile.user_id.in_(members))
    )

    # [
    #   ('01H88FWWRFSXMBGM9GYWM060HP', 'test1', None, ''),
    #   ('01H88EVDZY1R0E267XSB7XGXHG', 'htylight', 'cznowoyn1692502503.png', ''),
    #  ]
    await session.close()
    return group_chat, res.all()


async def update_group_name(group_chat_id: str, new_name: str):
    session = async_session()
    try:
        await session.execute(
            update(GroupChat).values(name=new_name).where(GroupChat.id == group_chat_id)
        )
        await session.commit()
    except Exception:
        raise Exception
    finally:
        await session.close()


async def update_group_intro(group_chat_id: str, new_intro: str):
    session = async_session()
    try:
        await session.execute(
            update(GroupChat)
            .values(introduction=new_intro)
            .where(GroupChat.id == group_chat_id)
        )
        await session.commit()
    except Exception:
        raise Exception
    finally:
        await session.close()


async def update_group_remark(user_id: str, group_chat_id: str, new_remark: str):
    session = async_session()
    try:
        contact: Contact = (
            await session.scalars(select(Contact).where(Contact.user_id == user_id))
        ).one()
        contact.group_chats[group_chat_id]["groupChatRemark"] = new_remark
        flag_modified(contact, "group_chats")
        session.add(contact)
        await session.commit()
    except Exception:
        raise Exception
    finally:
        await session.close()


async def update_my_remark(user_id: str, group_chat_id: str, new_my_remark: str):
    session = async_session()
    try:
        contact: Contact = (
            await session.scalars(select(Contact).where(Contact.user_id == user_id))
        ).one()
        contact.group_chats[group_chat_id]["remarkInGroupChat"] = new_my_remark
        flag_modified(contact, "group_chats")
        session.add(contact)
        await session.commit()
    except Exception:
        raise Exception
    finally:
        await session.close()


async def delete_group_chat(group_chat_id: str, members: list[str]):
    session = async_session()
    try:
        await session.execute(delete(GroupChat).where(GroupChat.id == group_chat_id))

        res: ScalarResult[Contact] = await session.scalars(
            select(Contact).where(Contact.user_id.in_(members))
        )

        for contact in res.all():
            del contact.group_chats[group_chat_id]
            flag_modified(contact, "group_chats")
            session.add(contact)

        await session.commit()
    except Exception:
        raise Exception
    finally:
        await session.close()


async def delete_group_chat_member(group_chat_id: str, member_id: str):
    session = async_session()
    try:
        group_chat: GroupChat = (
            await session.scalars(
                select(GroupChat).where(GroupChat.id == group_chat_id)
            )
        ).one()
        group_chat.members.remove(member_id)
        flag_modified(group_chat, "members")

        contact: Contact = (
            await session.scalars(select(Contact).where(Contact.user_id == member_id))
        ).one()
        del contact.group_chats[group_chat_id]
        flag_modified(contact, "group_chats")

        session.add_all([group_chat, contact])
        await session.commit()
    except Exception:
        raise Exception
    finally:
        session.close()
