from datetime import datetime, date
from enum import StrEnum

from sqlalchemy.dialects.postgresql import JSONB
from sqlalchemy.orm import DeclarativeBase, relationship
from sqlalchemy.orm import mapped_column, Mapped
from sqlalchemy import String, Integer, Enum, ARRAY, Date, Boolean
from sqlalchemy import ForeignKey

from .json_typeddict import *


class Base(DeclarativeBase):
    pass


class UserAccount(Base):
    __tablename__ = "user_account"

    id: Mapped[str] = mapped_column(String(26), primary_key=True)
    username: Mapped[str] = mapped_column(String(20), unique=True)
    email: Mapped[str] = mapped_column(String, unique=True)
    password: Mapped[str] = mapped_column(String(60))
    updated_at: Mapped[datetime] = mapped_column(default=datetime.now)
    created_at: Mapped[datetime] = mapped_column(
        default=datetime.now, onupdate=datetime.now
    )
    profile: Mapped["UserProfile"] = relationship(back_populates="user")
    contact: Mapped["Contact"] = relationship(back_populates="user")

    def __repr__(self):
        return f"UserAccount(username={self.username}, email={self.email})"

    def to_dict(self) -> dict:
        return {
            "id": self.id,
            "username": self.username,
            "email": self.email,
        }


class Gender(StrEnum):
    man = "man"
    woman = "woman"


class UserProfile(Base):
    __tablename__ = "user_profile"

    id: Mapped[int] = mapped_column(Integer, primary_key=True)
    nickname: Mapped[str] = mapped_column(String)
    gender: Mapped[Gender] = mapped_column(Enum(Gender), nullable=True)
    birthday: Mapped[date] = mapped_column(Date, nullable=True)
    location: Mapped[str] = mapped_column(String, nullable=True)
    status: Mapped[str] = mapped_column(String, nullable=True)
    sign: Mapped[str] = mapped_column(String, nullable=True)
    avatar: Mapped[str] = mapped_column(String, nullable=True)
    user_id: Mapped[str] = mapped_column(
        ForeignKey("user_account.id", ondelete="CASCADE")
    )
    user: Mapped["UserAccount"] = relationship(back_populates="profile")

    def __repr__(self):
        return f"UserProfile(user={self.user_id}," f"nickname={self.nickname})"

    def to_dict(self):
        return {
            "nickname": self.nickname,
            "gender": self.gender,
            "birthday": self.birthday and self.birthday.isoformat(),
            "location": self.location,
            "status": self.status,
            "sign": self.sign,
            "avatar": self.avatar,
        }


class Contact(Base):
    __tablename__ = "contact"

    id: Mapped[int] = mapped_column(Integer, primary_key=True)
    friends: Mapped[dict[str, FriendSetting]] = mapped_column(JSONB, nullable=True)
    friend_groups: Mapped[list[str]] = mapped_column(ARRAY(String), nullable=True)
    default_group: Mapped[str] = mapped_column(String, default="我的好友", nullable=True)
    group_chats: Mapped[dict[str, GroupChatSetting]] = mapped_column(
        JSONB, nullable=True
    )
    user_id: Mapped[str] = mapped_column(
        ForeignKey("user_account.id", ondelete="CASCADE")
    )
    user: Mapped["UserAccount"] = relationship(back_populates="contact")

    def __repr__(self):
        return (
            f"Contact("
            f"user={self.user_id}, "
            f"friends={self.friends}, "
            f"friend_group={self.friend_groups}, "
            f"default_group={self.default_group},"
            f"group_chats={self.group_chats}"
            f")"
        )

    def to_dict(self):
        return {
            "friends": self.friends,
            "friendGroups": self.friend_groups,
            "defaultGroup": self.default_group,
            "groupChats": self.group_chats,
        }


class Apply(Base):
    __tablename__ = "apply"

    id: Mapped[int] = mapped_column(Integer, primary_key=True)
    relation: Mapped[int] = mapped_column(Integer)
    applicant: Mapped[str] = mapped_column(String(26))
    recipient: Mapped[str] = mapped_column(String(26))
    group_chat_id: Mapped[str] = mapped_column(String(26), nullable=True)
    hello: Mapped[str] = mapped_column(String)
    setting: Mapped[dict] = mapped_column(JSONB)
    created_at: Mapped[datetime] = mapped_column(
        default=datetime.now, onupdate=datetime.now
    )

    def to_dict(self):
        return {
            "relation": self.relation,
            "applicant": self.applicant,
            "recipient": self.recipient,
            "groupChatId": self.group_chat_id,
            "hello": self.hello,
            "setting": self.setting,
            "createdAt": self.created_at.strftime("%y-%m-%d %H:%M:%S"),
        }


class GroupChat(Base):
    __tablename__ = "group_chat"
    id: Mapped[str] = mapped_column(String(11), primary_key=True)
    name: Mapped[str] = mapped_column(String)
    supervisor: Mapped[str] = mapped_column(String(26))
    administrators: Mapped[list[str]] = mapped_column(ARRAY(String(26)), default=[])
    members: Mapped[list[str]] = mapped_column(ARRAY(String(26)))
    introduction: Mapped[str] = mapped_column(String(100), default="")
    tags: Mapped[list[str]] = mapped_column(ARRAY(String), default=[])
    noticeboard: Mapped[list[dict]] = mapped_column(JSONB, default=[])
    avatar: Mapped[str] = mapped_column(String, default="")
    created_at: Mapped[datetime] = mapped_column(
        default=datetime.now, onupdate=datetime.now
    )

    def to_dict(self):
        return {
            "id": self.id,
            "name": self.name,
            "supervisor": self.supervisor,
            "administrators": self.administrators,
            "members": self.members,
            "introduction": self.introduction,
            "noticeboard": self.noticeboard,
            "tags": self.tags,
            "avatar": self.avatar,
        }


class UnreceivedMsg(Base):
    __tablename__ = "unreceived_msg"

    id: Mapped[int] = mapped_column(Integer, primary_key=True)
    msg_id: Mapped[str] = mapped_column(String(18))
    event: Mapped[str] = mapped_column(String)
    type: Mapped[str] = mapped_column(String)
    receiver_id: Mapped[str] = mapped_column(String(26))
    sender_id: Mapped[str] = mapped_column(String(26))
    group_chat_id: Mapped[str] = mapped_column(String(11), nullable=True)
    remark_in_group_chat: Mapped[str] = mapped_column(String, nullable=True)
    nickname: Mapped[str] = mapped_column(String, nullable=True)
    avatar: Mapped[str] = mapped_column(String, nullable=True)
    text: Mapped[str] = mapped_column(String)
    attachments: Mapped[list[str]] = mapped_column(ARRAY(String))
    date_time: Mapped[str] = mapped_column(String)
    is_show_time: Mapped[bool] = mapped_column(Boolean)

    def to_dict(self):
        return {
            "msgId": self.msg_id,
            "event": self.event,
            "type": self.type,
            "senderId": self.sender_id,
            "groupChatId": self.group_chat_id,
            "nickname": self.nickname,
            "remarkInGroupChat": self.remark_in_group_chat,
            "avatar": self.avatar,
            "text": self.text,
            "attachments": self.attachments,
            "dateTime": self.date_time,
            "isShowTime": self.is_show_time,
        }
