from fastapi import APIRouter
from pydantic import BaseModel

from ..crud import group_chat_crud
from ..response_models.group_chat_response import (
    BaseResponseModel,
    GroupChatProfileResponse,
    MemberNameAvatarResponse,
    FullProfileResponse,
)

router = APIRouter(prefix="/group_chat", tags=["group_chat"])


class GroupChatCreate(BaseModel):
    supervisor: str
    members: list[str]


class GroupChatMembers(BaseModel):
    group_chat_id: str
    members: list[str]


@router.post("/create", response_model=GroupChatProfileResponse)
async def create_group_chat(group_chat_create: GroupChatCreate):
    try:
        group_chat = await group_chat_crud.insert_group_chat(
            **group_chat_create.model_dump()
        )
        return {
            "code": 10800,
            "msg": "Create Group Chat Successfully",
            "data": group_chat.to_dict(),
        }

    except Exception as e:
        print(f"Creating Group Chat Fail With Error: {e}")
        return {"code": 9999, "msg": "Server Error"}


@router.post("/invite_members", response_model=BaseResponseModel)
async def invite_members(member_invitation: GroupChatMembers):
    try:
        await group_chat_crud.insert_group_chat_members(
            **member_invitation.model_dump()
        )
        return {"code": 10800, "msg": "Invite Members Successfully"}
    except Exception as e:
        print(f"Invite members fail with error: {e}")
        return {"code": 9999, "msg": "Server Error"}


@router.get("/member_name_avatar", response_model=MemberNameAvatarResponse)
async def get_member_name_avatar(group_chat_id: str, member_id: str, is_friend: bool):
    res = await group_chat_crud.select_member_name_avatar(member_id, is_friend)
    print(res)
    data = {}
    if is_friend:
        if res.get(group_chat_id):
            # make sure my friend is still in this group chat
            data["remark"] = res[group_chat_id]["myRemark"]
        data["nickname"] = ""
        data["avatar"] = ""
    else:
        if len(res) == 3:
            # make sure this user is still in this group chat
            data["remark"] = res[2][group_chat_id]["myRemark"]
        data["nickname"] = res[0]
        data["avatar"] = res[1] or ""

    return {
        "code": 10800,
        "msg": "Get Group Chat Member Name and Avatar Successfully",
        "data": data,
    }


@router.get("/full_profile", response_model=FullProfileResponse)
async def get_full_profile(group_chat_id: str):
    # full profile, includes introduction, noticeboard, group avatar,
    # tags, and all members remark, avatar
    group_chat, member_name_avatar_list = await group_chat_crud.select_full_profile(
        group_chat_id
    )

    member_name_avatar_dict = {}

    for member_name_avatar in member_name_avatar_list:
        member_name_avatar_dict[member_name_avatar[0]] = {
            "remark": member_name_avatar[3],
            "nickname": member_name_avatar[1],
            "avatar": member_name_avatar[2] or "",
        }
    print(group_chat.to_dict())
    print(member_name_avatar_dict)
    return {
        "code": 10800,
        "msg": "Get Group Chat Full Profile Successfully",
        "groupChat": group_chat.to_dict(),
        "memberNameAvatar": member_name_avatar_dict,
    }


@router.post("/change_name", response_model=BaseResponseModel)
async def change_name(group_chat_id: str, new_name: str):
    try:
        await group_chat_crud.update_group_name(group_chat_id, new_name)
        return {"code": 10800, "msg": "Change Group Chat Name Successfully"}
    except Exception as e:
        print(f"Change Group Chat Name Fail With Error: {e}")
        return {"code": 9999, "msg": "Server Error"}


@router.post("/change_intro", response_model=BaseResponseModel)
async def change_intro(group_chat_id: str, new_intro: str):
    try:
        await group_chat_crud.update_group_intro(group_chat_id, new_intro)
        return {"code": 10800, "msg": "Change Group Chat Intro Successfully"}
    except Exception as e:
        print(f"Change Group Chat Intro Fail With Error: {e}")
        return {"code": 9999, "msg": "Server Error"}


@router.post("/change_remark", response_model=BaseResponseModel)
async def change_name_remark(user_id: str, group_chat_id: str, new_remark: str):
    try:
        await group_chat_crud.update_group_remark(user_id, group_chat_id, new_remark)
        return {"code": 10800, "msg": "Change Group Chat Remark Successfully"}
    except Exception as e:
        print(f"Change Group Chat Remark Fail With Error: {e}")
        return {"code": 9999, "msg": "Server Error"}


@router.post("/change_my_remark", response_model=BaseResponseModel)
async def change_my_remark(user_id: str, group_chat_id: str, new_my_remark: str):
    try:
        await group_chat_crud.update_my_remark(user_id, group_chat_id, new_my_remark)
        return {"code": 10800, "msg": "Change My Nickname in Group Chat Successfully"}
    except Exception as e:
        print(f"Change Group Chat My Nickname in Group Chat Fail With Error: {e}")
        return {"code": 9999, "msg": "Server Error"}


@router.post("/delete", response_model=BaseResponseModel)
async def delete_group_chat(deletion: GroupChatMembers):
    try:
        await group_chat_crud.delete_group_chat(**deletion.model_dump())
        return {"code": 10800, "msg": "Delete Group Chat Successfully"}
    except Exception as e:
        print(f"Delete Group Chat Fail With Error: {e}")
        return {"code": 9999, "msg": "Server Error"}


@router.post("/quit", response_model=BaseResponseModel)
async def quit_group_chat(group_chat_id: str, member_id: str):
    try:
        await group_chat_crud.delete_group_chat_member(group_chat_id, member_id)
        return {"code": 10800, "msg": "Quit Group Chat Successfully"}
    except Exception as e:
        print(f"Quit Group Chat Fail With Error: {e}")
        return {"code": 9999, "msg": "Server Error"}
