import threading
import asyncio

from fastapi import APIRouter, WebSocket, WebSocketDisconnect, BackgroundTasks, Depends
from pydantic import BaseModel

from ..dependencies import verify_token
from ..utils.web_socket import WebSocketManager
from ..utils.static_file import read_chat_file, async_read_chat_file
from ..crud import unreceived_msg_crud
from ..database.models import UnreceivedMsg
from ..response_models.message_response import UnreceivedMsgResponse

router = APIRouter(tags=["message"])

ws_manager = WebSocketManager()

loop = asyncio.new_event_loop()


async def push_unsent_messages():
    i = 0
    while True:
        print(i)
        print(ws_manager.active_socket)
        i += 1
        for user_id in ws_manager.active_socket:
            try:
                msgs: list[UnreceivedMsg] = unreceived_msg_crud.select_msgs(user_id)
                for msg in msgs:
                    await ws_manager.send_to_another(user_id, msg.to_dict())
                    for attachment in msg.attachments:
                        for (
                            current_chunk_num,
                            total_chunk_num,
                            byte_array,
                        ) in read_chat_file(attachment):
                            await ws_manager.send_to_another(
                                user_id,
                                {
                                    "event": "chat-image",
                                    "filename": attachment,
                                    "tempFilename": f"temp/{attachment}-${total_chunk_num}-${current_chunk_num}",
                                    "totalChunkNum": total_chunk_num,
                                    "currentChunkNum": current_chunk_num,
                                    "bytes": byte_array,
                                },
                            )
                if msgs:
                    unreceived_msg_crud.delete_msgs(user_id)
            except Exception as e:
                print(e)

        await asyncio.sleep(30)


# https://blog.csdn.net/hellojike/article/details/115092039
def start_loop_thread(loop):
    asyncio.set_event_loop(loop)
    loop.run_forever()


polling_message_thread = threading.Thread(target=start_loop_thread, args=(loop,))


@router.on_event("startup")
def message_startup_event():
    polling_message_thread.start()
    asyncio.run_coroutine_threadsafe(push_unsent_messages(), loop)


@router.on_event("shutdown")
async def message_shutdown_event():
    print("关闭所有连接............")
    loop.stop()
    ws_manager.disconnect_all()


@router.websocket("/ws/{user_id}")
async def connect_websocket(websocket: WebSocket, user_id: str, is_reconnect: bool):
    if is_reconnect:
        await ws_manager.reconnect(user_id, websocket)
    else:
        await ws_manager.connect(user_id, websocket)
    try:
        while True:
            data = await websocket.receive_json()

            match data["event"]:
                case "ping":
                    await ws_manager.active_socket[user_id].send_json({"type": "pong"})
                case "friend-chat-msg":
                    await ws_manager.send_to_another(data["receiverId"], data)
                    if len(data["attachments"]) > 0:
                        await ws_manager.send_to_another(
                            data["senderId"],
                            {
                                "event": "pull-chat-image",
                                "chatType": 0,
                                "attachments": data["attachments"],
                                "receiverId": data["receiverId"],
                            },
                        )
                case "apply-friend":
                    await ws_manager.send_to_another(data["recipient"], data)
                case "friend-added" | "friend-deleted":
                    await ws_manager.send_to_another(data["receiverId"], data)
                case "chat-image":
                    if data.get("receiverId"):
                        await ws_manager.send_to_another(data["receiverId"], data)
                    else:
                        receiver_ids = data["receiverIds"]
                        del data["receiverIds"]
                        await ws_manager.broadcast(receiver_ids, data)
                case "group-chat-creation":
                    receiver_ids = data["receiverIds"]
                    del data["receiverIds"]
                    await ws_manager.broadcast(receiver_ids, data)
                case "group-chat-msg":
                    receiver_ids = data["receiverIds"]
                    del data["receiverIds"]
                    await ws_manager.broadcast(receiver_ids, data)
                    if len(data["attachments"]) > 0:
                        await ws_manager.send_to_another(
                            data["senderId"],
                            {
                                "event": "pull-chat-image",
                                "chatType": 1,
                                "attachments": data["attachments"],
                                "receiverIds": receiver_ids,
                            },
                        )

    except WebSocketDisconnect:
        print(f"{user_id} disconnect")
        ws_manager.disconnect(user_id)


@router.get(
    "/message/unreceived",
    response_model=UnreceivedMsgResponse,
    dependencies=[Depends(verify_token)],
)
async def get_unreceived_msgs(
    receiver_id: str,
    background_tasks: BackgroundTasks,
):
    try:
        msgs: list[UnreceivedMsg] = await unreceived_msg_crud.delete_and_return_msgs(
            receiver_id
        )
        msgs.sort(key=lambda msg: msg.msg_id)
        json_msgs = []
        all_msg_attachments = []

        for msg in msgs:
            if not msg.attachments:
                all_msg_attachments.extend(msg.attachments)
            json_msgs.append(msg.to_dict())

        if all_msg_attachments:
            background_tasks.add_task(
                send_image_by_websocket,
                receiver_id,
                all_msg_attachments,
            )

        return {
            "code": 10900,
            "msg": "Get Unreceived Messages Successfully",
            "data": json_msgs,
        }
    except Exception as e:
        print(e)
        return {"code": 9999, "msg": "Server Error"}


class UploadAttachment(BaseModel):
    event: str
    senderId: str
    receiverId: str | None = None
    receiverIds: list[str] | None = None
    filename: str
    tempFilename: str
    totalChunkNum: int
    currentChunkNum: int
    bytes: list[int]


@router.post("/message/attachment")
async def upload_attachment(data: UploadAttachment):
    data = data.model_dump()
    if data.get("receiverId"):
        await ws_manager.send_to_another(data["receiverId"], data)
    else:
        await ws_manager.broadcast(data["receiverIds"], data)

    return {"code": 10900, "msg": "Ok"}


async def send_image_by_websocket(receiver_id: str, attachments: list[str]):
    for attachment in attachments:
        for (
            current_chunk_num,
            total_chunk_num,
            byte_array,
        ) in await async_read_chat_file(attachment):
            await ws_manager.send_to_another(
                receiver_id,
                {
                    "event": "chat-image",
                    "filename": attachment,
                    "tempFilename": f"temp/{attachment}-${total_chunk_num}-${current_chunk_num}",
                    "totalChunkNum": total_chunk_num,
                    "currentChunkNum": current_chunk_num,
                    "bytes": byte_array,
                },
            )
